//+------------------------------------------------------------------+
//|                                                  Correlation.mq4 |
//|                                     Copyright  2009, Apparelink |
//|                                        http://www.apparelink.com |
//+------------------------------------------------------------------+
#property copyright "Copyright  2009, Apparelink"
#property link      "http://www.apparelink.com"
#property library
#include <StringArraySort.mqh>

//+------------------------------------------------------------------+
//| Finds any character(s) and replaces it with another in a string.
//|
//| REQUIREMENTS
//| The standard StringTrimRight() and StringTrimLeft() functions
//| cannot be used because they trim only the end part of a string
//| rather than remove all matches from the entire string.
//+------------------------------------------------------------------+
string FindAndReplace(string text, string matched_text, string replace_with_text)
{
   int start_pos, end_pos;
   string text1, text2, temp_text;
   
   while (StringFind(text, matched_text, 0) != -1)
   {  
      //Example: AAA**BBB to AAABBB requires FindAndReplace("AAA**BBB", "**", "")
      start_pos = StringFind(text, matched_text, 0); //Get the first position of the first occurence of the match - 3
      end_pos = start_pos + StringLen(matched_text) - 1; //Get the last position of the first occurence of the match - 3+2-1=4
      text1 = StringSubstr(text, 0, start_pos); //Extract the first part of the string before the match - "AAA"
      text2 = StringSubstr(text, end_pos + 1); //Extract the second part of the string after the match - "BBB"
      temp_text = text1; //"AAA"
      temp_text = temp_text + replace_with_text; // "AAA" + ""
      temp_text = temp_text + text2; // "AAA" + "" + "BBB"
      text = temp_text; //"AAABBB"
   }
   return (text);
}

//+------------------------------------------------------------------+
//| Splits a delimited string into array elements.
//|
//| REQUIREMENTS
//| The function will check if the input string is valid and trim any
//| padding at both ends (e.g., "*") in order to convert it into a
//| proper delimited string before processing into an array.
//|
//| For example, the following asterisk delimited strings will result
//| in the same string array:
//|
//| BEFORE
//| "***XAUUSD*AUDUSD*GBPUSD*"
//| "*XAUUSD*AUDUSD*GBPUSD"
//| "XAUUSD*AUDUSD*GBPUSD*"
//| "XAUUSD*AUDUSD*GBPUSD"
//|
//| AFTER
//| array[0] = "XAUUSD"
//| array[1] = "AUDUSD"
//| array[2] = "AUDUSD"
//+------------------------------------------------------------------+
void Split(string text, string delimiter, string &array[])
{
   /* Check if the input text is valid */
   if (StringLen(text) <= 0)
      return;
   
   /* Trim the padding characters at both ends */
   text = StringTrimLeftPadding(text, delimiter);  //"***XAUUSD*AUDUSD*GBPUSD*" to "XAUUSD*AUDUSD*GBPUSD*"
   text = StringReverse(text);                     //"XAUUSD*AUDUSD*GBPUSD*" to "*DSUPBG*DSUDUA*DSUUAX"
   text = StringTrimLeftPadding(text, delimiter);  //"*DSUPBG*DSUDUA*DSUUAX" to "DSUPBG*DSUDUA*DSUUAX"
   text = StringReverse(text);                     //"DSUPBG*DSUDUA*DSUUAX" to "XAUUSD*AUDUSD*GBPUSD"
   
   /* Extract values from a delimited string into an array */
   if (StringFind(text, delimiter) != -1) //Delimiter found in the string
   {
      int i = -1, delimiter_pos, element_count;    //element_count = StringMatchCount(text, delimiter) + 1;
      //ArrayResize(array, element_count);
      string delimiter_separated_value, new_delimiter_separated_values;
      //for (int i = 0; i < element_count; i++)
      while (StringFind(text, delimiter) != -1)
      {
         //Example 1: AAA**BBB**CCC with "**" being the delimiter
         delimiter_pos = StringFind(text, delimiter, 0); //3 is returned
         delimiter_separated_value = StringSubstr(text, 0, delimiter_pos); //"AAA" is extracted
         i++;
         element_count = i + 1;
         ArrayResize(array, element_count);         
         array[i] = delimiter_separated_value;
         new_delimiter_separated_values = StringSubstr(text, delimiter_pos + StringLen(delimiter)); //Unprocessed part becomes "BBB**CCC"
         text = new_delimiter_separated_values;
      }
      array[i+1] = text; //Element after the last delimiter
   }
   else //Delimiter not found
   {
      ArrayResize(array, 1);  
      array[0] = text; //Return the text unchanged
   }
}

//+------------------------------------------------------------------+
//| Reverses all characters in the string.
//+------------------------------------------------------------------+
string StringReverse(string text)
{
   string temp_str;
   for (int i = StringLen(text)-1; i >= 0; i--)
      temp_str = temp_str + StringSubstr(text, i, 1);
   return (temp_str);
}

//+------------------------------------------------------------------+
//| Removes the padding at both ends of the string.
//|
//| REQUIREMENTS
//| The standard StringTrimRight() and StringTrimLeft() functions
//| cannot be used because they only remove line feed characters,
//| spaces and tabs rather than user-specified characters.
//|
//| This function will produce the same string after removing the
//| padding characters from the input string.  For example,
//|
//| BEFORE
//| "***XAUUSD*AUDUSD*AUDUSD*GBPUSD*"
//| "*XAUUSD*AUDUSD*AUDUSD*GBPUSD"
//|
//| AFTER
//| "XAUUSD*AUDUSD*AUDUSD*GBPUSD"
//+------------------------------------------------------------------+
string StringTrimLeftPadding(string text, string padding)
{
   string temp_str = text;
   int next_pos, padding_pos, padding_len = StringLen(padding);
   while ((StringFind(temp_str, padding) != -1) &&             //Padding characters found
          (StringSubstr(temp_str, 0, padding_len) == padding)) //More leading characters to remove
   {
      padding_pos = StringFind(temp_str, padding);
      next_pos = padding_pos + padding_len;
      temp_str = StringSubstr(temp_str, next_pos);
   }
   return (temp_str);
}

//+------------------------------------------------------------------+
//| Gets the next delimited value.
//+------------------------------------------------------------------+
string GetNextDelimiterSeparatedValue()
{

}

//+------------------------------------------------------------------+
//| Returns the number of matches within a string.
//|
//| REQUIREMENTS
//| This function is necessary because StringFind() only returns
//| the position of the first match.
//+------------------------------------------------------------------+
int StringMatchCount(string text, string matched_text)
{
   int start_pos = 1, counter = 0;
   while (StringFind(text, matched_text, 0) != -1)
   {
      if (start_pos <= StringLen(text))
      {
         start_pos = StringFind(text, matched_text, 0) + StringLen(matched_text); //Next position after the first match
         counter++;
         text = StringSubstr(text, start_pos);
      }
   }
   return (counter);
}

//+------------------------------------------------------------------+
//| Converts all characters in a string into upper case.
//+------------------------------------------------------------------+
string StringUpperCase(string text)
{
   string   temp_str = text;
   int      char,
            len = StringLen(text) - 1;
   while (len >= 0)
   {
      char = StringGetChar(temp_str, len);
      if((char > 96 && char < 123) || (char > 223 && char < 256))
               temp_str = StringSetChar(temp_str, len, char - 32);
       else 
           if(char > -33 && char < 0)
               temp_str = StringSetChar(temp_str, len, char + 224);
      len--;
   }
   return(temp_str);
}
//+------------------------------------------------------------------+
//| Checks if the element is in the array.
//|
//| REQUIREMENTS
//| The function returns true if the first element is found in the
//| array, otherwise false.
//+------------------------------------------------------------------+
bool Exists(string array[], string element)
{
/* Elements in an array is unsorted, a sequential search
   from top to bottom is required.
*/   
   int i, size = ArraySize(array);
   for (i = 0; i < size; i++)
      if (array[i] == element)
         return (true);
   return (false);
}

//+------------------------------------------------------------------+
//| Copies non-null elements from a source array to the other.
//+------------------------------------------------------------------+
void ArrayCopyNonNull(string &dest[], string source[])
{
   int j = -1, count, size = ArraySize(source);
   ArrayResize(dest, size); //Use the non-null element count to resize destination array
   for (int i = 0; i < size; i++)
      if (StringLen(source[i]) != 0)
      {
         j++;
         dest[j] = source[i];
         count = j + 1;
      }
   ArrayResize(dest, count);
}

//+------------------------------------------------------------------+
//| Copies only unique elements from a source array to the other.
//| The destination array size may be the same or smaller than that
//| of the source array.
//|
//| NOTE
//| Technique used to extract the unique elements requires the array
//| be sorted first.
//+------------------------------------------------------------------+
void ArrayCopyUnique(string &dest[], string source[])
{
   int size = ArraySize(source);
   ArrayResize(dest, size);
   StringArraySort(source, MODE_ASCEND); //From "ABCBCACBA" to "AAABBBCCC"
   int i = -1, j = -1, count;
   i++;
   j++;
   dest[j] = source[i]; //Copy the first element from a sorted array to a new one
   for (; i < size-1; i++) //Copy remaining elements
      if (source[i] != source[i+1])
      {
         j++;
         dest[j] = source[i+1];
         count = j + 1;
      }
   ArrayResize(dest, count);
}

//+------------------------------------------------------------------+
//| Counts non-null elements in an array.
//+------------------------------------------------------------------+
int GetNonNullElementCount(string array[])
{
   int count = 0, size = ArraySize(array);
   for (int i = 0; i < size; i++)
      if (StringLen(array[i]) != 0)
         count++;
   return (count);
}

//+------------------------------------------------------------------+
//| Counts unique elements in an array.
//+------------------------------------------------------------------+
int GetUniqueElementCount(string array[])
{
   string temp[];
   ArrayCopy(temp, array); //Example: "ABCBCACBA"
   StringArraySort(temp, MODE_ASCEND); //From "ABCBCACBA" to "AAABBBCCC"
   int count = 0, size = ArraySize(temp);
   for (int i = 0; i < size-1; i++)
      if (temp[i] != temp[i+1])
         count++;
   return (count+1);
}

//+------------------------------------------------------------------+
//| Sorts the array elements in ascending order.
//| 1 - Ascending (default)
//| 2 - Descending
//+------------------------------------------------------------------+
void StringArraySort(string &array[], int sort_dir = MODE_ASCEND)
{
   int count = ArraySize(array);
   switch(sort_dir)
   {  
      case MODE_ASCEND  : SortStrArrayRange(array, 0, count);
      case MODE_DESCEND : SortStrArrayRange(array, count, 0);
   }
}

//+------------------------------------------------------------------+
//| Calculates the standard deviation of a set of values in an array.
//| MathStdev() measures how widely values are dispersed from the
//| average value (the mean).
//+------------------------------------------------------------------+
double MathStdev(double &array[])
{
   int size = ArraySize(array);
   double sum,
          stdev,
          mean = MathAvg(array);
   for (int i = 0; i < size; i++)
      sum += MathPow((array[i] - mean), 2);
   stdev = MathSqrt(sum / (size -1));
   return (stdev);
}

//+------------------------------------------------------------------+
//| Calculates the average value (the mean) of a set of values in an
//| array.
//+------------------------------------------------------------------+
double MathAvg(double &array[])
{
   int size = ArraySize(array);
   double sum,
          mean;
   for (int i = 0; i < size; i++)
      sum += array[i];
   mean = sum / size;
   return (mean);
}

//+------------------------------------------------------------------+
//| Calculates the sum of all values in an array.
//+------------------------------------------------------------------+
double MathSum(double &array[])
{
   int size = ArraySize(array);
   double sum;
   for (int i = 0; i < size; i++)
      sum += array[i];
   return (sum);
}

//+------------------------------------------------------------------+
//| Calculates the sum of all values squared in an array.
//+------------------------------------------------------------------+
double MathSumSqrt(double &array[])
{
   int size = ArraySize(array);
   double sum;
   for (int i = 0; i < size; i++)
      sum += MathPow(array[i], 2);
   return (sum);
}

//+------------------------------------------------------------------+
//| Calculates the sum of the products.
//+------------------------------------------------------------------+
double MathSumProd(double &array1[], double &array2[])
{
   /* Check array size */
   if (ArraySize(array1) != ArraySize(array2))
   {
      Print("MathSumProd(): Two arrays must be the same size.");
      return (0);
   }
   else
      int size = ArraySize(array1);   
   double sum;
   for (int i = 0; i < size; i++)
      sum += array1[i] * array2[i];
   return (sum);
}